#+PROPERTY: header-args :results none
#+TITLE: Emacs should become a Wayland compositor

* Setup
#+begin_src emacs-lisp
(defvar ew-talk-p)
(defvar ew-talk-buffer)
(setq ew-talk-p t
      ew-talk-buffer (current-buffer))

(defun ew-talk-switch (arg)
  (interactive "N")
  (with-current-buffer ew-talk-buffer
    (widen)
    (pcase arg
      (-1 (outline-previous-heading))
      (1 (outline-next-heading)))
    (org-narrow-to-subtree)
    ;; (setq header-line-format (org-get-heading))
    (forward-line)
    (when (org-in-src-block-p)
      (org-babel-execute-src-block))
    (forward-line -1)
    ;; (when ew-talk-p
    ;;   (narrow-to-region (point) (point-max)))
    ))

(defun ew-talk-prev ()
  (interactive)
  (if (string= "Setup" (org-get-heading))
      (message "Already at first slide")
    (ew-talk-switch -1)))

(defun ew-talk-next ()
  (interactive)
  (ew-talk-switch 1))

(let ((map (make-sparse-keymap)))
  (define-key map [left] #'ew-talk-prev)
  (define-key map [right] #'ew-talk-next)
  (set-transient-map map (lambda () ew-talk-p)))

(defvar ew-talk-mpv nil)
(defvar ew-talk-mpv-buffer "*X video0 - mpv*")
(unless (process-live-p ew-talk-mpv)
  (setq ew-talk-mpv
        (make-process
         :name "webcam"
         :command (list "mpv" "--input-ipc-server=/tmp/mpvsocket" "--profile=low-latency" "--untimed" "/dev/video0")
         :buffer nil)))

(org-show-all)
(org-hide-block-all)
;; (setq text-scale-remap-header-line t)
(text-scale-set 3)
(ew-talk-next)
#+end_src

* Record
#+begin_src emacs-lisp
;; (eshell)
(split-window-right)
(switch-to-buffer "*ansi-term*")
#+end_src

* Intro
#+begin_src emacs-lisp
;; Show Video buffer fullscreen
(delete-other-windows)
(switch-to-buffer ew-talk-mpv-buffer)
(shell-command "echo 'show-text \"Emacs should become a Wayland compositor\" 30000' | socat - /tmp/mpvsocket")
#+end_src

* Hello EmacsConf :)
#+begin_src emacs-lisp
(delete-other-windows)
(switch-to-buffer ew-talk-buffer)
(split-window-right)
(other-window 1)
(split-window-below)
(goto-char (point-max))
(recenter)
(other-window 1)
(switch-to-buffer "-")
(split-window-right 21)
(other-window 1)
(switch-to-buffer ew-talk-mpv-buffer)
(redisplay)
(other-window 1)
#+end_src


     Emacs should become
             a 
     Wayland compositor

     
     # ~ EXWM & more






And it already kind of is now ;)
* Why
#+begin_src emacs-lisp
(delete-other-windows)
(split-window-right -50)
(other-window 1)
(switch-to-buffer "-")
(split-window-below -16)
(other-window 1)
(switch-to-buffer ew-talk-mpv-buffer)
(other-window 1)
#+end_src



- Emacs can do wayland now (pgtk)

- Bigger toolbox

- Can be standalone (DRI)
  OR Nested under X | Wayland

- I want to keep living in Emacs
* EXWM usecase
#+begin_src emacs-lisp
(delete-other-windows)
(split-window-right -50)
(other-window 1)
(switch-to-buffer "-")
(split-window-below -16)
(other-window 1)
(switch-to-buffer ew-talk-mpv-buffer)
(other-window 1)
#+end_src



- Wayland surface inside emacs window

- Handle input & clipboard

- Lispyfy the Linux Desktop
* Xwidget usecase
#+begin_src emacs-lisp
(delete-other-windows)
(ewb-buffer-float ew-talk-mpv-buffer 750 250 444 222)
#+end_src

- Embed anything that can create a wayland surface

- Video, web, 3d, opengl ... 
* How
#+begin_src emacs-lisp
(ewb-buffer-float ew-talk-mpv-buffer 900 380 350 150)
#+end_src

- Blocker: Emacs & Wayland server have own event loop
  -> Make Emacs speak Wayland

- Use wlroots for minimal wayland server
  do rest in Emacs

- ews :: Wayland Server in C       1000 loc
- ewc :: Wayland Client in Elisp    300 loc
- ewp :: Wayland Protocol
- ewb :: Wayland buffers            550 loc

still buggy &  missing input handling
* Caveats


- wlroots is 60 kloc & in active development

- wlr_scene can't crop yet

- Once Wayland is enabled it stays on 
  or GTK kills Emacs too

- Bit of work & fuzzing ahead
  
* Call to action

- A further step to Emacs OS
  - Output & input handling

- Use it in more ways for emacs display?
  Wayland manages simple pixel buffers
  Go back to the old X ways & maybe ditch GTK?

- Emacs Wayland protocol allows concise design
  Improve on EXWM Kiss-Style

* Thank you






-> perma-curious.eu

* Quit
#+begin_src emacs-lisp
(switch-to-buffer ew-talk-mpv-buffer)
(redisplay)
(switch-to-buffer ew-talk-buffer)
(text-scale-set 0)
(widen)
(org-show-block-all)
;; (setq header-line-format nil)
(setq ew-talk-p nil)
#+end_src
